@@ -164,7 +164,12 @@ static int compute_cross_correlation(int16_t *s1, int n1, int16_t *s2_padded, fl
 	for (i=0; i<xcorr_nsamples; i++){
 		norm2 += s2_padded[step*(i+n1-1)]*s2_padded[step*(i+n1-1)];
 		tmp = scalar_product(s1, s2_padded + i*step, n1, step);
-		xcorr[i] = (float)((double)tmp / sqrt((double)(norm1)*(double)norm2));
+		double square_n1n2 = sqrt((double)(norm1)*(double)norm2);
+		if (square_n1n2 > 0)
+			xcorr[i] = (float)((double)tmp / square_n1n2);
+		else
+			xcorr[i] = 1;
+			
 		tmp = tmp < 0 ? -tmp : tmp;
 		if (tmp > max){
 			max = tmp;
@@ -206,7 +211,7 @@ static int _ms_audio_diff_one_chunk(int16_t *s1, int16_t *s2, int nsamples, int
 		
 		max_pos = 0;
 		/*sum the square of r and l xcorr signals to determine the global maximum*/
-		for (i = 0; i < max_shift_samples; ++i){
+		for (i = 0; i <= max_shift_samples; ++i){// max_shift_samples is takken account because of the correlation computation that compute the shift at the shift index and not shift-1. At 0, there is no shift then, for next index we add step*(n1+i) and remove the base reference s2_padded[step*i] each times : The scope is moved. So, the max_shift_samples index is included to the max computation.
 			xcorr_r[i] = xcorr_r[i]*xcorr_r[i] + xcorr_l[i]*xcorr_l[i];
 			if (xcorr_r[i] > max){
 				max = xcorr_r[i];
