@@ -19,17 +19,23 @@
 
 #include "mediastreamer2/msconference.h"
 #include "mediastreamer2/msaudiomixer.h"
+#include "mediastreamer2/msvolume.h"
 #include "private.h"
 
+static const int audio_threshold_min_db = -30;
+
 struct _MSAudioConference{
 	MSTicker *ticker;
 	MSFilter *mixer;
 	MSAudioConferenceParams params;
+	bctbx_list_t *members; /* list of MSAudioEndpoint */
 	int nmembers;
+	MSAudioEndpoint *active_speaker;
 };
 
 struct _MSAudioEndpoint{
 	AudioStream *st;
+	void *user_data;
 	MSFilter *in_resampler,*out_resampler;
 	MSCPoint out_cut_point;
 	MSCPoint in_cut_point;
@@ -38,9 +44,11 @@ struct _MSAudioEndpoint{
 	MSCPoint mixer_out;
 	MSAudioConference *conference;
 	MSFilter *recorder; /* in case it is a recorder endpoint*/
+	MSFilter *recorder_encoder; /* in case the recorder is mkv */
 	MSFilter *player; /* not used at the moment, but we need it so that there is a source connected to the mixer*/
 	int pin;
 	int samplerate;
+	bool_t muted;
 };
 
 
@@ -149,7 +157,9 @@ static void plumb_to_conf(MSAudioEndpoint *ep){
 		out_rate=in_rate=ep->samplerate;
 	}else in_rate=out_rate=conf->params.samplerate;
 	
-	if (ep->recorder){
+	if (ep->recorder_encoder){
+		ms_filter_call_method(ep->recorder_encoder,MS_FILTER_SET_SAMPLE_RATE,&conf->params.samplerate);
+	}else if (ep->recorder){
 		ms_filter_call_method(ep->recorder,MS_FILTER_SET_SAMPLE_RATE,&conf->params.samplerate);
 	}
 	
@@ -178,7 +188,9 @@ void ms_audio_conference_add_member(MSAudioConference *obj, MSAudioEndpoint *ep)
 	if (obj->nmembers>0) ms_ticker_detach(obj->ticker,obj->mixer);
 	plumb_to_conf(ep);
 	ms_ticker_attach(obj->ticker,obj->mixer);
+	obj->members = bctbx_list_append(obj->members, ep);
 	obj->nmembers++;
+	ms_audio_conference_mute_member(obj, ep, ep->muted);
 }
 
 static void unplumb_from_conf(MSAudioEndpoint *ep){
@@ -199,6 +211,7 @@ void ms_audio_conference_remove_member(MSAudioConference *obj, MSAudioEndpoint *
 	unplumb_from_conf(ep);
 	ep->conference=NULL;
 	obj->nmembers--;
+	obj->members = bctbx_list_remove(obj->members, ep);
 	if (obj->nmembers>0) ms_ticker_attach(obj->ticker,obj->mixer);
 }
 
@@ -206,6 +219,7 @@ void ms_audio_conference_mute_member(MSAudioConference *obj, MSAudioEndpoint *ep
 	MSAudioMixerCtl ctl={0};
 	ctl.pin=ep->pin;
 	ctl.param.active=!muted;
+	ep->muted = muted;
 	ms_filter_call_method(ep->conference->mixer, MS_AUDIO_MIXER_SET_ACTIVE, &ctl);
 }
 
@@ -214,19 +228,44 @@ int ms_audio_conference_get_size(MSAudioConference *obj){
 }
 
 
+void ms_audio_conference_process_events(MSAudioConference *obj){
+	const bctbx_list_t *elem;
+	float max_db_over_member = MS_VOLUME_DB_LOWEST;
+	MSAudioEndpoint *winner = NULL;
+	
+	for (elem = obj->members; elem != NULL; elem = elem->next){
+		MSAudioEndpoint *ep = (MSAudioEndpoint *) elem->data;
+		int is_remote;
+		if (ep->st == NULL) continue; /* This happens for the player/recorder special endpoint */
+		is_remote = (ep->in_cut_point_prev.filter == ep->st->volrecv);
+		MSFilter *volume_filter = is_remote ? ep->st->volrecv : ep->st->volsend;
+		if (ep->muted) continue;
+		if (volume_filter){
+			float max_db = MS_VOLUME_DB_LOWEST;
+			if (ms_filter_call_method(volume_filter, MS_VOLUME_GET_MAX, &max_db) == 0){
+				if (max_db > audio_threshold_min_db && max_db > max_db_over_member){
+					max_db_over_member = max_db;
+					winner = ep;
+				}
+			}
+			
+		}
+	}
+	if (obj->active_speaker != winner && winner != NULL){
+		ms_message("Active speaker changed: now on pin %i", winner->pin);
+		if (obj->params.active_talker_callback)
+			obj->params.active_talker_callback(obj, winner);
+		obj->active_speaker = winner;
+	}
+}
+
+
 void ms_audio_conference_destroy(MSAudioConference *obj){
 	ms_ticker_destroy(obj->ticker);
 	ms_filter_destroy(obj->mixer);
 	ms_free(obj);
 }
 
-//MSAudioEndpoint *ms_audio_endpoint_new(void){
-//	MSAudioEndpoint *ep=ms_new0(MSAudioEndpoint,1);
-//	ep->in_resampler=ms_filter_new(MS_RESAMPLE_ID);
-//	ep->out_resampler=ms_filter_new(MS_RESAMPLE_ID);
-//	ep->samplerate=8000;
-//	return ep;
-//}
 
 MSAudioEndpoint *ms_audio_endpoint_new(void){
 	MSAudioEndpoint *ep=ms_new0(MSAudioEndpoint,1);
@@ -235,6 +274,14 @@ MSAudioEndpoint *ms_audio_endpoint_new(void){
 	return ep;
 }
 
+void ms_audio_endpoint_set_user_data(MSAudioEndpoint *ep, void *user_data){
+	ep->user_data = user_data;
+}
+
+void * ms_audio_endpoint_get_user_data(const MSAudioEndpoint *ep){
+	return ep->user_data;
+}
+
 MSAudioEndpoint * ms_audio_endpoint_get_from_stream(AudioStream *st, bool_t is_remote){
 	MSAudioEndpoint *ep=ms_audio_endpoint_new();
 	ep->st=st;
@@ -252,35 +299,58 @@ void ms_audio_endpoint_release_from_stream(MSAudioEndpoint *obj){
 void ms_audio_endpoint_destroy(MSAudioEndpoint *ep){
 	if (ep->in_resampler) ms_filter_destroy(ep->in_resampler);
 	if (ep->out_resampler) ms_filter_destroy(ep->out_resampler);
+	if (ep->recorder_encoder){
+		ms_filter_unlink(ep->recorder_encoder, 0, ep->recorder, 0);
+		ms_filter_destroy(ep->recorder_encoder);
+	}
 	if (ep->recorder) ms_filter_destroy(ep->recorder);
 	if (ep->player) ms_filter_destroy(ep->player);
 	ms_free(ep);
 }
 
-MSAudioEndpoint * ms_audio_endpoint_new_recorder(MSFactory* factory){
+
+MSAudioEndpoint * ms_audio_endpoint_new_recorder(MSFactory* factory, const char *path){
 	MSAudioEndpoint *ep=ms_audio_endpoint_new();
+	
+	if (ms_path_ends_with(path,".mkv")){
+		MSPinFormat pinfmt = {0};
+		
+		ep->recorder_encoder = ms_factory_create_filter(factory, MS_OPUS_ENC_ID);
+		ep->recorder = ms_factory_create_filter(factory, MS_MKV_RECORDER_ID);
+		ms_filter_link(ep->recorder_encoder, 0, ep->recorder, 0);
+		
+		pinfmt.pin = 0;
+		pinfmt.fmt = ms_factory_get_audio_format(factory, "opus", 48000, 2, NULL);
+		ms_filter_call_method(ep->recorder, MS_FILTER_SET_INPUT_FMT, &pinfmt);
+	}else if (ms_path_ends_with(path, ".wav")){
+		ep->recorder=ms_factory_create_filter(factory, MS_FILE_REC_ID);
+	}else{
+		ms_error("Unsupported audio file extension for path %s .", path);
+		ms_audio_endpoint_destroy(ep);
+		return NULL;
+	}
+	ms_filter_call_method(ep->recorder,MS_RECORDER_OPEN,(void*)path);
+	
 	ep->in_resampler=ms_factory_create_filter(factory, MS_RESAMPLE_ID);
 	ep->out_resampler=ms_factory_create_filter(factory, MS_RESAMPLE_ID);
-	ep->recorder=ms_factory_create_filter(factory, MS_FILE_REC_ID);
 	ep->player=ms_factory_create_filter(factory, MS_FILE_PLAYER_ID);
-	ep->mixer_out.filter=ep->recorder;
+	ep->mixer_out.filter= ep->recorder_encoder ? ep->recorder_encoder : ep->recorder;
 	ep->mixer_in.filter=ep->player;
 	ep->samplerate=-1;
 	return ep;
 }
 
-int ms_audio_recorder_endpoint_start(MSAudioEndpoint *ep, const char *path){
-	int err;
+int ms_audio_recorder_endpoint_start(MSAudioEndpoint *ep){
 	MSRecorderState state;
 	if (!ep->recorder){
 		ms_error("This endpoint isn't a recorder endpoint.");
 		return -1;
 	}
 	ms_filter_call_method(ep->recorder,MS_RECORDER_GET_STATE,&state);
-	if (state!=MSRecorderClosed)
-		ms_filter_call_method_noarg(ep->recorder,MS_RECORDER_CLOSE);
-	err=ms_filter_call_method(ep->recorder,MS_RECORDER_OPEN,(void*)path);
-	if (err==-1) return -1;
+	if (state != MSRecorderPaused){
+		ms_error("Recorder not bad state, cannot start.");
+		return -1;
+	}
 	return ms_filter_call_method_noarg(ep->recorder,MS_RECORDER_START);
 }
 
