@@ -37,7 +37,9 @@
 #include "mediastreamer2/msvaddtx.h"
 #include "mediastreamer2/msgenericplc.h"
 #include "mediastreamer2/mseventqueue.h"
+#include "mediastreamer2/flowcontrol.h"
 #include "private.h"
+#include <math.h>
 
 #ifdef __ANDROID__
 #include "mediastreamer2/devices.h"
@@ -91,6 +93,8 @@ static void audio_stream_free(AudioStream *stream) {
 	if (stream->outbound_mixer) ms_filter_destroy(stream->outbound_mixer);
 	if (stream->recorder_file) ms_free(stream->recorder_file);
 	if (stream->rtp_io_session) rtp_session_destroy(stream->rtp_io_session);
+	if (stream->captcard) ms_snd_card_unref(stream->captcard);
+	if (stream->playcard) ms_snd_card_unref(stream->playcard);
 
 	ms_free(stream);
 }
@@ -171,6 +175,46 @@ static bool_t audio_stream_payload_type_changed(RtpSession *session, void *data)
 	return FALSE;
 }
 
+/*
+ * note: Only AAudio, OpenSLES, and IOS leverage internal ID for input streams.
+ */
+static int audio_stream_configure_input_snd_card(AudioStream *stream) {
+	MSSndCard *card = stream->captcard;
+	int ok = -1;
+	if (stream->soundread) {
+		if(ms_filter_implements_interface(stream->soundread, MSFilterAudioCaptureInterface) ) {
+			if( ms_filter_has_method(stream->soundread,MS_AUDIO_CAPTURE_SET_INTERNAL_ID) ) {
+				ms_filter_call_method(stream->soundread, MS_AUDIO_CAPTURE_SET_INTERNAL_ID, card);
+				ms_message("[AudioStream] set input sound card for %s:%p to %s", ms_filter_get_name(stream->soundread), stream->soundread, card->id);
+				ok = 0;
+			}else{
+				ms_warning("[AudioStream] MS_AUDIO_CAPTURE_SET_INTERNAL_ID is not implemented, cannot set input card for %s:%p to %s", ms_filter_get_name(stream->soundread), stream->soundread, card->id);
+			}
+		}
+	}
+	return ok;
+}
+
+/*
+ * note: Only AAudio, OpenSLES, and IOS leverage internal ID for output streams.
+ */
+static int audio_stream_configure_output_snd_card(AudioStream *stream) {
+	MSSndCard *card = stream->playcard;
+	int ok = -1;
+	if (stream->soundwrite) {
+		if(ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+			if( ms_filter_has_method(stream->soundwrite,MS_AUDIO_PLAYBACK_SET_INTERNAL_ID) ){
+				ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_INTERNAL_ID, card);
+				ms_message("[AudioStream] set output sound card for %s:%p to %s", ms_filter_get_name(stream->soundwrite), stream->soundwrite, card->id);
+				ok = 0;
+			}else{
+				ms_warning("[AudioStream] MS_AUDIO_PLAYBACK_SET_INTERNAL_ID is not implemented, cannot set output card for %s:%p to %s", ms_filter_get_name(stream->soundread), stream->soundread, card->id);
+			}
+		}
+	}
+	return ok;
+}
+
 /*
  * note: since not all filters implement MS_FILTER_GET_SAMPLE_RATE and MS_FILTER_GET_NCHANNELS, the PayloadType passed here is used to guess this information.
  */
@@ -181,20 +225,25 @@ static void audio_stream_configure_resampler(AudioStream *st, MSFilter *resample
 	ms_filter_call_method(to,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
 	ms_filter_call_method(from, MS_FILTER_GET_NCHANNELS, &from_channels);
 	ms_filter_call_method(to, MS_FILTER_GET_NCHANNELS, &to_channels);
+
+	// Access name member only if filter desc member is not null to aviod segfaults
+	const char * from_name = (from) ? ((from->desc) ? from->desc->name : "Unknown") : "Unknown";
+	const char * to_name = (to) ? ((to->desc) ? to->desc->name : "Unknown" ) : "Unknown";
+
 	if (from_channels == 0) {
 		from_channels = st->nchannels;
-		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", from->desc->name);
+		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", from_name);
 	}
 	if (to_channels == 0) {
 		to_channels = st->nchannels;
-		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", to->desc->name);
+		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", to_name);
 	}
 	if (from_rate == 0){
-		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", from->desc->name);
+		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", from_name);
 		from_rate = st->sample_rate;
 	}
 	if (to_rate == 0){
-		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", to->desc->name);
+		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", to_name);
 		to_rate = st->sample_rate;
 	}
 	ms_filter_call_method(resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
@@ -203,7 +252,7 @@ static void audio_stream_configure_resampler(AudioStream *st, MSFilter *resample
 	ms_filter_call_method(resampler, MS_FILTER_SET_OUTPUT_NCHANNELS, &to_channels);
 	ms_message(
 		"configuring %s:%p-->%s:%p from rate [%i] to rate [%i] and from channel [%i] to channel [%i]",
-		from->desc->name, from, to->desc->name, to, from_rate, to_rate, from_channels, to_channels
+		from_name, from, to_name, to, from_rate, to_rate, from_channels, to_channels
 	);
 }
 
@@ -414,17 +463,18 @@ static float audio_stream_get_rtcp_xr_average_lq_quality_rating(void *userdata)
 	return audio_stream_get_average_lq_quality_rating(stream);
 }
 
-static bool_t ci_ends_with(const char *filename, const char*suffix){
-	size_t filename_len=strlen(filename);
+bool_t ms_path_ends_with(const char* path, const char* suffix){
+	size_t filename_len=strlen(path);
 	size_t suffix_len=strlen(suffix);
 	if (filename_len<suffix_len) return FALSE;
-	return strcasecmp(filename+filename_len-suffix_len,suffix)==0;
+	return strcasecmp(path+filename_len-suffix_len,suffix)==0;
 }
 
+
 MSFilter *_ms_create_av_player(const char *filename, MSFactory* factory){
-	if (ci_ends_with(filename,".mkv"))
+	if (ms_path_ends_with(filename,".mkv"))
 		return ms_factory_create_filter(factory, MS_MKV_PLAYER_ID);
-	else if (ci_ends_with(filename,".wav"))
+	else if (ms_path_ends_with(filename,".wav"))
 		return ms_factory_create_filter(factory, MS_FILE_PLAYER_ID);
 	else
 		ms_error("Cannot open %s, unsupported file extension", filename);
@@ -813,8 +863,17 @@ static int get_usable_telephone_event(RtpProfile *profile, int clock_rate){
 	return fallback_pt;
 }
 
+static void ms_audio_flow_control_event_handler(void *user_data, MSFilter *source, unsigned int event, void *eventdata) {
+	if (event == MS_AUDIO_FLOW_CONTROL_DROP_EVENT) {
+		MSFilter *flow_controller = (MSFilter *)user_data;
+		MSAudioFlowControlDropEvent *ev = (MSAudioFlowControlDropEvent *)eventdata;
+		ms_filter_call_method(flow_controller, MS_AUDIO_FLOW_CONTROL_DROP, ev);
+	}
+}
+
 int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
 	const char *rem_rtcp_ip, int rem_rtcp_port, int payload, const MSMediaStreamIO *io) {
+
 	RtpSession *rtps=stream->ms.sessions.rtp_session;
 	PayloadType *pt;
 	int tmp, tev_pt;
@@ -866,9 +925,15 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 
 	/* creates the local part */
 	if (io->input.type == MSResourceSoundcard){
+		MSSndCard * card = io->input.soundcard;
 		if (stream->soundread==NULL)
-			stream->soundread = ms_snd_card_create_reader(io->input.soundcard);
+			stream->soundread = ms_snd_card_create_reader(card);
 		has_builtin_ec=!!(ms_snd_card_get_capabilities(io->input.soundcard) & MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER);
+		if (stream->captcard) {
+			ms_snd_card_unref(stream->captcard);
+			stream->captcard = NULL;
+		}
+		stream->captcard = ms_snd_card_ref(card);
 	} else if (io->input.type == MSResourceRtp) {
 		stream->rtp_io_session = io->input.session;
 		pt = rtp_profile_get_payload(rtp_session_get_profile(stream->rtp_io_session),
@@ -880,8 +945,14 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		stream->soundread=ms_factory_create_filter(stream->ms.factory, MS_FILE_PLAYER_ID);
 	}
 	if (io->output.type == MSResourceSoundcard) {
+		MSSndCard * card = io->output.soundcard;
 		if (stream->soundwrite==NULL)
-			stream->soundwrite=ms_snd_card_create_writer(io->output.soundcard);
+			stream->soundwrite=ms_snd_card_create_writer(card);
+		if (stream->playcard) {
+			ms_snd_card_unref(stream->playcard);
+			stream->playcard = NULL;
+		}
+		stream->playcard = ms_snd_card_ref(card);
 	} else if (io->output.type == MSResourceRtp) {
 		stream->rtp_io_session = io->output.session;
 		pt = rtp_profile_get_payload(rtp_session_get_profile(stream->rtp_io_session),
@@ -948,8 +1019,10 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 	/* be able to use the echo canceller wich may be limited (webrtc aecm max frequency is 16000 Hz) */
 	// First check if we need to use the echo canceller
 	// Overide feature if not requested or done at sound card level
-	if ( ((stream->features & AUDIO_STREAM_FEATURE_EC) && !stream->use_ec) || has_builtin_ec )
+	if ( ((stream->features & AUDIO_STREAM_FEATURE_EC) && !stream->use_ec) || has_builtin_ec ){
+		ms_message("Software echo cancellation disabled: use_ec=%i, has_builtin_ec=%i", stream->use_ec, has_builtin_ec);
 		stream->features &=~AUDIO_STREAM_FEATURE_EC;
+	}
 
 	/*configure the echo canceller if required */
 	if ((stream->features & AUDIO_STREAM_FEATURE_EC) == 0 && stream->ec != NULL) {
@@ -1037,7 +1110,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 	ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels);
 
 	if (stream->ec){
-		if (!stream->is_ec_delay_set) {
+		if (!stream->is_ec_delay_set && io->input.soundcard) {
 			int delay_ms=ms_snd_card_get_minimal_latency(io->input.soundcard);
 			ms_message("Setting echo canceller delay with value provided by soundcard: %i ms",delay_ms);
 			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&delay_ms);
@@ -1310,14 +1383,14 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec){
 	MSMediaStreamIO io = MS_MEDIA_STREAM_IO_INITIALIZER;
 
-	if (playcard){
+	if (playcard) {
 		io.output.type = MSResourceSoundcard;
 		io.output.soundcard = playcard;
 	}else{
 		io.output.type = MSResourceFile;
 		io.output.file = outfile;
 	}
-	if (captcard){
+	if (captcard) {
 		io.input.type = MSResourceSoundcard;
 		io.input.soundcard = captcard;
 	}else{
@@ -1528,6 +1601,7 @@ AudioStream *audio_stream_new_with_sessions(MSFactory *factory, const MSMediaStr
 	stream->use_agc=FALSE;
 	stream->use_ng=FALSE;
 	stream->features=AUDIO_STREAM_FEATURE_ALL;
+	stream->disable_record_on_mute=FALSE;
 
 	rtp_session_set_rtcp_xr_media_callbacks(stream->ms.sessions.rtp_session, &rtcp_xr_media_cbs);
 
@@ -1544,6 +1618,7 @@ AudioStream *audio_stream_new2(MSFactory* factory, const char* ip, int loc_rtp_p
 	sessions.rtp_session=ms_create_duplex_rtp_session(ip,loc_rtp_port,loc_rtcp_port, ms_factory_get_mtu(factory));
 	obj=audio_stream_new_with_sessions(factory, &sessions);
 	obj->ms.owns_sessions=TRUE;
+	obj->last_mic_gain_level_db = 0;
 	return obj;
 }
 
@@ -1605,12 +1680,29 @@ void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val){
 	}
 }
 
+void audio_stream_enable_mic(AudioStream *stream, bool_t enabled) {
+	if (stream->soundread) {
+		if (stream->disable_record_on_mute && ms_filter_has_method(stream->soundread, MS_AUDIO_CAPTURE_MUTE)) {
+			bool_t muted = !enabled;
+			ms_filter_call_method(stream->soundread, MS_AUDIO_CAPTURE_MUTE, &muted);
+		}
+	}
+		
+	if (enabled)
+		audio_stream_set_mic_gain_db(stream, stream->last_mic_gain_level_db);
+	else
+		audio_stream_set_mic_gain(stream, 0);
+}
+
 void audio_stream_set_mic_gain_db(AudioStream *stream, float gain_db) {
 	audio_stream_set_rtp_output_gain_db(stream, gain_db);
 }
 
 void audio_stream_set_mic_gain(AudioStream *stream, float gain){
-	if (stream->volsend){
+	if (stream->volsend) {
+		if (gain!=0) {
+			stream->last_mic_gain_level_db = 10*ortp_log10f(gain);
+		}
 		ms_filter_call_method(stream->volsend,MS_VOLUME_SET_GAIN,&gain);
 	}else ms_warning("Could not apply gain: gain control wasn't activated. "
 			"Use audio_stream_enable_gain_control() before starting the stream.");
@@ -1808,6 +1900,15 @@ void audio_stream_stop(AudioStream * stream){
 			}
 			/*dismantle the remote play part*/
 			close_av_player(stream);
+
+			if (stream->captcard) {
+				ms_snd_card_unref(stream->captcard);
+				stream->captcard = NULL;
+			}
+			if (stream->playcard) {
+				ms_snd_card_unref(stream->playcard);
+				stream->playcard = NULL;
+			}
 		}
 	}
 	rtp_session_set_rtcp_xr_media_callbacks(stream->ms.sessions.rtp_session, NULL);
@@ -1857,6 +1958,7 @@ static void audio_stream_set_rtp_output_gain_db(AudioStream *stream, float gain_
 #endif
 
 	if (stream->volsend){
+		stream->last_mic_gain_level_db = gain_db;
 		ms_filter_call_method(stream->volsend, MS_VOLUME_SET_DB_GAIN, &gain);
 	} else ms_warning("Could not apply gain on sent RTP packets: gain control wasn't activated. "
 			"Use audio_stream_enable_gain_control() before starting the stream.");
@@ -1919,10 +2021,11 @@ float audio_stream_get_average_lq_quality_rating(AudioStream *stream) {
 }
 
 void audio_stream_enable_zrtp(AudioStream *stream, MSZrtpParams *params){
-	if (stream->ms.sessions.zrtp_context==NULL)
+	if (stream->ms.sessions.zrtp_context==NULL) {
 		stream->ms.sessions.zrtp_context=ms_zrtp_context_new( &(stream->ms.sessions), params);
-	else if (!media_stream_secured(&stream->ms))
+	} else if (!media_stream_secured(&stream->ms)) {
 		ms_zrtp_reset_transmition_timer(stream->ms.sessions.zrtp_context);
+	}
 }
 
 void audio_stream_start_zrtp(AudioStream *stream) {
@@ -1978,3 +2081,31 @@ void audio_stream_set_audio_route(AudioStream *stream, MSAudioRoute route) {
 		}
 	}
 }
+
+int audio_stream_set_input_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_capture) {
+	MSSndCard* captcard = ms_snd_card_ref(sndcard_capture);
+	if (stream->captcard) {
+		ms_snd_card_unref(stream->captcard);
+		stream->captcard = NULL;
+	}
+	stream->captcard = captcard;
+	return audio_stream_configure_input_snd_card(stream);
+}
+
+int audio_stream_set_output_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_playback) {
+	MSSndCard* playcard = ms_snd_card_ref(sndcard_playback);
+	if (stream->playcard) {
+		ms_snd_card_unref(stream->playcard);
+		stream->playcard = NULL;
+	}
+	stream->playcard = playcard;
+	return audio_stream_configure_output_snd_card(stream);
+}
+
+MSSndCard * audio_stream_get_input_ms_snd_card(AudioStream *stream) {
+	return stream->captcard;
+}
+
+MSSndCard * audio_stream_get_output_ms_snd_card(AudioStream *stream) {
+	return stream->playcard;
+}
