@@ -108,10 +108,16 @@ static MSFileFormat four_cc_to_file_format(const FourCC four_cc) {
 MSMediaPlayer *ms_media_player_new(MSFactory* factory, MSSndCard *snd_card, const char *video_display_name, void *window_id) {
 	MSMediaPlayer *obj = (MSMediaPlayer *)ms_new0(MSMediaPlayer, 1);
 	ms_mutex_init(&obj->cb_access, NULL);
-	obj->snd_card = snd_card;
-	if(video_display_name != NULL && strlen(video_display_name) > 0) {
+	obj->snd_card = ms_snd_card_ref(snd_card);
+	if (video_display_name != NULL && strlen(video_display_name) > 0) {
 		obj->video_display = ms_strdup(video_display_name);
 		obj->window_id = window_id;
+#ifdef ANDROID
+		if (obj->window_id) {
+			JNIEnv *env = ms_get_jni_env();
+			obj->window_id = (*env)->NewGlobalRef(env, obj->window_id);
+		}
+#endif
 	}
 	obj->factory = factory;
 	obj->loop_interval = -1;
@@ -120,6 +126,14 @@ MSMediaPlayer *ms_media_player_new(MSFactory* factory, MSSndCard *snd_card, cons
 
 void ms_media_player_free(MSMediaPlayer *obj) {
 	ms_media_player_close(obj);
+	ms_snd_card_unref(obj->snd_card);
+#ifdef ANDROID
+	if (obj->window_id) {
+		JNIEnv *env = ms_get_jni_env();
+		(*env)->DeleteGlobalRef(env, obj->window_id);
+		obj->window_id = NULL;
+	}
+#endif
 	ms_free_if_not_null(obj->video_display);
 	ms_free(obj);
 }
@@ -128,6 +142,26 @@ void * ms_media_player_get_window_id(const MSMediaPlayer *obj) {
 	return obj->window_id;
 }
 
+void ms_media_player_set_window_id(MSMediaPlayer *obj, void *window_id) {
+#ifdef ANDROID
+	if (obj->window_id) {
+		JNIEnv *env = ms_get_jni_env();
+		(*env)->DeleteGlobalRef(env, obj->window_id);
+		obj->window_id = NULL;
+	}
+#endif
+	obj->window_id = window_id;
+#ifdef ANDROID
+	if (obj->window_id) {
+		JNIEnv *env = ms_get_jni_env();
+		obj->window_id = (*env)->NewGlobalRef(env, obj->window_id);
+	}
+#endif
+	if (obj->video_sink) {
+		ms_filter_call_method(obj->video_sink, MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID, &obj->window_id);
+	}
+}
+
 bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath) {
 	wave_header_t header;
 	int fd;
@@ -197,6 +231,10 @@ bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath) {
 	return TRUE;
 }
 
+bool_t ms_media_player_get_is_video_available(MSMediaPlayer *obj) {
+	return obj->video_sink != NULL;
+}
+
 void ms_media_player_close(MSMediaPlayer *obj) {
 	if(obj->is_open) {
 		ms_message("MSMediaPlayer: closing file.");
@@ -417,10 +455,10 @@ static void _create_sinks(MSMediaPlayer *obj) {
 			ms_error("Could not create audio sink. Soundcard=%s", obj->snd_card->name);
 		}
 	}
-	if(obj->video_pin_fmt.fmt) {
-		if(obj->video_display) {
+	if (obj->video_pin_fmt.fmt) {
+		if (obj->video_display) {
 			obj->video_sink = ms_factory_create_filter_from_name(obj->factory, obj->video_display);
-			if(obj->video_sink) {
+			if (obj->video_sink) {
 				if(obj->window_id) ms_filter_call_method(obj->video_sink, MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID, &obj->window_id);
 			} else {
 				ms_error("Could not create video sink: %s", obj->video_display);
