@@ -136,6 +136,16 @@ void video_stream_free(VideoStream *stream) {
 	ms_free(stream);
 }
 
+static void source_event_cb(void *ud, MSFilter* f, unsigned int event, void *eventdata){
+	VideoStream *st=(VideoStream*)ud;
+	switch (event) {// Allow a source to reinitialize all tree formats
+		case MS_FILTER_OUTPUT_FMT_CHANGED:
+				video_stream_update_video_params(st);
+			break;
+		default:{}
+	}
+}
+
 static void event_cb(void *ud, MSFilter* f, unsigned int event, void *eventdata){
 	VideoStream *st=(VideoStream*)ud;
 	if (st->eventcb!=NULL){
@@ -176,6 +186,10 @@ static void internal_event_cb(void *ud, MSFilter *f, unsigned int event, void *e
 	}
 }
 
+static void video_stream_process_encoder_control(VideoStream *stream, unsigned int method_id, void *arg, void *user_data){
+	ms_filter_call_method(stream->ms.encoder, method_id, arg);
+}
+
 static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 	VideoStream *stream = (VideoStream *)media_stream;
 	int i;
@@ -191,7 +205,7 @@ static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 				if (rtcp_fb_fir_fci_get_ssrc(fci) == rtp_session_get_send_ssrc(stream->ms.sessions.rtp_session)) {
 					uint8_t seq_nr = rtcp_fb_fir_fci_get_seq_nr(fci);
 					/* TODO: manage seq_nr and ignore FIR repeats to avoid flooding the encoder */
-					ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_NOTIFY_FIR, &seq_nr);
+					stream->encoder_control_cb(stream,  MS_VIDEO_ENCODER_NOTIFY_FIR, &seq_nr, stream->encoder_control_cb_user_data);
 					stream->ms_video_stat.counter_rcvd_fir++;
 					ms_message("Got RTCP FIR on video stream [%p] SSRC [%x] count %d, seq %u", 
 						stream, rtcp_fb_fir_fci_get_ssrc(fci), stream->ms_video_stat.counter_rcvd_fir, seq_nr);
@@ -212,7 +226,7 @@ static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 				case RTCP_PSFB_PLI:
 
 					stream->ms_video_stat.counter_rcvd_pli++;
-					ms_filter_call_method_noarg(stream->ms.encoder, MS_VIDEO_ENCODER_NOTIFY_PLI);
+					stream->encoder_control_cb(stream, MS_VIDEO_ENCODER_NOTIFY_PLI, NULL, stream->encoder_control_cb_user_data);
 					ms_message("Got RTCP PLI on video stream [%p] SSRC [%x] count %d", 
 						stream, rtcp_PSFB_get_media_source_ssrc(m), stream->ms_video_stat.counter_rcvd_pli);
 					break;
@@ -225,7 +239,7 @@ static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 						sli.first = rtcp_fb_sli_fci_get_first(fci);
 						sli.number = rtcp_fb_sli_fci_get_number(fci);
 						sli.picture_id = rtcp_fb_sli_fci_get_picture_id(fci);
-						ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_NOTIFY_SLI, &sli);
+						stream->encoder_control_cb(stream, MS_VIDEO_ENCODER_NOTIFY_SLI, &sli, stream->encoder_control_cb_user_data);
 						stream->ms_video_stat.counter_rcvd_sli++;
 						ms_message("video_stream_process_rtcp stream [%p] SLI count %d", stream,  stream->ms_video_stat.counter_rcvd_sli);
 
@@ -237,7 +251,7 @@ static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 					MSVideoCodecRPSI rpsi;
 					rpsi.bit_string = rtcp_fb_rpsi_fci_get_bit_string(fci);
 					rpsi.bit_string_len = rtcp_PSFB_rpsi_get_fci_bit_string_len(m);
-					ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_NOTIFY_RPSI, &rpsi);
+					stream->encoder_control_cb(stream, MS_VIDEO_ENCODER_NOTIFY_RPSI, &rpsi, stream->encoder_control_cb_user_data);
 					stream->ms_video_stat.counter_rcvd_rpsi++;
 					ms_message("video_stream_process_rtcp stream [%p] RPSI count %d", stream,  stream->ms_video_stat.counter_rcvd_rpsi);
 				}
@@ -255,6 +269,15 @@ static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
 	}
 }
 
+void video_stream_set_encoder_control_callback(VideoStream *stream, VideoStreamEncoderControlCb cb, void *user_data){
+	if (cb == NULL) {
+		cb = video_stream_process_encoder_control;
+		user_data = NULL;
+	}
+	stream->encoder_control_cb = cb;
+	stream->encoder_control_cb_user_data = user_data;
+}
+
 static void stop_preload_graph(VideoStream *stream){
 	ms_ticker_detach(stream->ms.sessions.ticker,stream->ms.rtprecv);
 	ms_filter_unlink(stream->ms.rtprecv,0,stream->ms.voidsink,0);
@@ -419,6 +442,7 @@ VideoStream *video_stream_new_with_sessions(MSFactory* factory, const MSMediaStr
 	stream->output_performs_decoding = FALSE;
 	choose_display_name(stream);
 	stream->ms.process_rtcp=video_stream_process_rtcp;
+	video_stream_set_encoder_control_callback(stream, NULL, NULL);
 	/*
 	 * In practice, these filters are needed only for audio+video recording.
 	 */
@@ -582,8 +606,8 @@ static MSVideoSize get_with_same_orientation_and_ratio(MSVideoSize size, MSVideo
 }
 #endif
 
-static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, bool_t first_time){
-	MSVideoSize cam_vsize;
+static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, bool_t source_changed){
+	MSVideoSize cam_vsize = {320, 240};
 	MSVideoConfiguration vconf;
 	MSPixFmt format=MS_PIX_FMT_UNKNOWN;
 	MSVideoEncoderPixFmt encoder_supports_source_format;
@@ -592,8 +616,9 @@ static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, boo
 	MSPinFormat pf={0};
 	bool_t is_player=ms_filter_get_id(stream->source)==MS_ITC_SOURCE_ID || ms_filter_get_id(stream->source)==MS_MKV_PLAYER_ID;
 
-	if (first_time) {
+	if (source_changed) {
 		ms_filter_add_notify_callback(stream->source, event_cb, stream, FALSE);
+		if (!is_player) ms_filter_add_notify_callback(stream->source, source_event_cb, stream, FALSE);
 		/* It is important that the internal_event_cb is called synchronously! */
 		ms_filter_add_notify_callback(stream->source, internal_event_cb, stream, TRUE);
 	}
@@ -646,6 +671,7 @@ static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, boo
 			//Use the size returned by the above call to MS_VIDEO_ENCODER_GET_CONFIGURATION directly
 			cam_vsize = vconf.vsize;
 		} else {
+			vconf.vsize = preview_vsize;
 			ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&preview_vsize);
 			/*the camera may not support the target size and suggest a one close to the target */
 			ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&cam_vsize);
@@ -697,12 +723,13 @@ static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, boo
 	}
 	stream->configured_fps=vconf.fps;
 
-	ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SET_CONFIGURATION, &vconf);
+	if(stream->ms.encoder)
+	    ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SET_CONFIGURATION, &vconf);
 
 	encoder_supports_source_format.supported = FALSE;
 	encoder_supports_source_format.pixfmt = format;
 
-	if (ms_filter_has_method(stream->ms.encoder, MS_VIDEO_ENCODER_SUPPORTS_PIXFMT) == TRUE) {
+	if (stream->ms.encoder && ms_filter_has_method(stream->ms.encoder, MS_VIDEO_ENCODER_SUPPORTS_PIXFMT) == TRUE) {
 		ret = ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SUPPORTS_PIXFMT, &encoder_supports_source_format);
 	} else {
 		ret = -1;
@@ -740,7 +767,7 @@ static void configure_video_source(VideoStream *stream, bool_t skip_bitrate, boo
 		ms_bitrate_controller_destroy(stream->ms.rc);
 		stream->ms.rc=NULL;
 	}
-	if (stream->ms.rc_enable){
+	if (stream->ms.rc_enable && stream->ms.encoder){
 		switch (stream->ms.rc_algorithm){
 		case MSQosAnalyzerAlgorithmSimple:
 			stream->ms.rc=ms_av_bitrate_controller_new(NULL,NULL,stream->ms.sessions.rtp_session,stream->ms.encoder);
@@ -903,6 +930,9 @@ int video_stream_start_from_io(VideoStream *stream, RtpProfile *profile, const c
 						ms_filter_call_method_noarg(source, MS_PLAYER_START);
 				}
 			break;
+			case MSResourceVoid:
+				stream->source = ms_factory_create_filter(stream->ms.factory, MS_VOID_SOURCE_ID);
+			break;
 			default:
 				ms_error("Unhandled input resource type %s", ms_resource_type_to_string(io->input.type));
 			break;
@@ -928,6 +958,9 @@ int video_stream_start_from_io(VideoStream *stream, RtpProfile *profile, const c
 				ms_filter_add_notify_callback(recorder, video_recorder_handle_event, stream, TRUE);
 				if (io->output.file) video_stream_open_remote_record(stream, io->output.file);
 			break;
+			case MSResourceVoid:
+				output = ms_factory_create_filter(stream->ms.factory, MS_VOID_SINK_ID);
+			break;
 			default:
 				/*will just display in all other cases*/
 				/*ms_error("Unhandled output resource type %s", ms_resource_type_to_string(io->output.type));*/
@@ -985,21 +1018,25 @@ static void apply_video_preset(VideoStream *stream, PayloadType *pt) {
 static void apply_bitrate_limit(VideoStream *stream, PayloadType *pt) {
 	int target_upload_bandwidth = 0;
 
-	if (stream->ms.max_target_bitrate <= 0) {
-		if (pt->normal_bitrate <= 0) {
-			ms_message("target and payload bitrates not set for stream [%p] using lowest configuration of preferred video size %dx%d",stream,stream->sent_vsize.width, stream->sent_vsize.height);
-		} else {
-			stream->ms.max_target_bitrate=pt->normal_bitrate;
-			ms_message("target bitrate not set for stream [%p] using payload's bitrate is %i",stream,stream->ms.max_target_bitrate);
-		}
+	if (stream->ms.target_bitrate > 0){
+		ms_message("Current target bitrate is set to [%i] bit/s.", stream->ms.target_bitrate);
+		target_upload_bandwidth = stream->ms.target_bitrate;
+	}
+	else if (pt->normal_bitrate <= 0 && stream->ms.max_target_bitrate <= 0) {
+		ms_message("target and payload bitrates not set for stream [%p] using lowest configuration of preferred video size %dx%d",stream,stream->sent_vsize.width, stream->sent_vsize.height);
+	} else if (pt->normal_bitrate > 0){
+		ms_message("Max target bitrate not set for stream [%p], but using payload type's bitrate [%i]",stream,stream->ms.max_target_bitrate);
+		target_upload_bandwidth = stream->ms.max_target_bitrate = pt->normal_bitrate;
+	}else{
+		ms_message("Using max target bitrate [%i] bit/s", stream->ms.max_target_bitrate); 
+		target_upload_bandwidth = stream->ms.max_target_bitrate;
 	}
 
-	ms_message("Limiting bitrate of video encoder to %i bits/s for stream [%p]",stream->ms.max_target_bitrate,stream);
 	if (stream->vconf_list != NULL) {
 		MSVideoConfiguration vconf;
 
-		if (stream->ms.max_target_bitrate > 0) {
-			vconf = ms_video_find_best_configuration_for_bitrate(stream->vconf_list, stream->ms.max_target_bitrate, ms_factory_get_cpu_count(stream->ms.factory));
+		if (target_upload_bandwidth > 0) {
+			vconf = ms_video_find_best_configuration_for_bitrate(stream->vconf_list, target_upload_bandwidth, ms_factory_get_cpu_count(stream->ms.factory));
 			/* Adjust configuration video size to use the user preferred video size if it is lower that the configuration one. */
 			if ((stream->sent_vsize.height * stream->sent_vsize.width) < (vconf.vsize.height * vconf.vsize.width)) {
 				vconf.vsize = stream->sent_vsize;
@@ -1008,18 +1045,19 @@ static void apply_bitrate_limit(VideoStream *stream, PayloadType *pt) {
 			/* We retrieve the lowest configuration for that vsize since the bandwidth estimator will increase quality if possible */
 			vconf = ms_video_find_worst_configuration_for_size(stream->vconf_list, stream->sent_vsize, ms_factory_get_cpu_count(stream->ms.factory));
 			/*If the lower config is found, required_bitrate will be 0. In this case, use the bitrate_limit*/
-			target_upload_bandwidth = vconf.required_bitrate > 0 ? vconf.required_bitrate : vconf.bitrate_limit;
+			vconf.required_bitrate = target_upload_bandwidth = vconf.required_bitrate > 0 ? vconf.required_bitrate : vconf.bitrate_limit;
 		}
+		ms_message("Limiting bitrate of video encoder to %i bits/s for stream [%p]", target_upload_bandwidth, stream);
 		ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SET_CONFIGURATION, &vconf);
 	} else {
 		MSVideoConfiguration vconf;
 		/* Get current video configuration and change only bitrate */
 		ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION, &vconf);
 
-		vconf.required_bitrate = stream->ms.max_target_bitrate;
+		vconf.required_bitrate = target_upload_bandwidth;
 		ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SET_CONFIGURATION, &vconf);
 	}
-	rtp_session_set_target_upload_bandwidth(stream->ms.sessions.rtp_session, target_upload_bandwidth != 0 ? target_upload_bandwidth : stream->ms.max_target_bitrate);
+	rtp_session_set_target_upload_bandwidth(stream->ms.sessions.rtp_session, target_upload_bandwidth);
 }
 
 static MSPixFmt mime_type_to_pix_format(const char *mime_type) {
@@ -1161,8 +1199,7 @@ static int video_stream_start_with_source_and_output(VideoStream *stream, RtpPro
 			if (stream->preview_window_id!=0){
 				ms_filter_call_method(stream->output2, MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID,&stream->preview_window_id);
 			}
-
-			if (ms_filter_get_id(stream->output2) == MS_OGL_ID) {
+			if( ms_filter_implements_interface(stream->output2, MSFilterVideoDisplayInterface) ) {
 				assign_value_to_mirroring_flag_to_preview(stream);
 			}
 			ms_filter_link(stream->tee,1,stream->output2,0);
@@ -1176,10 +1213,7 @@ static int video_stream_start_with_source_and_output(VideoStream *stream, RtpPro
 	if (output != NULL) {
 		rtp_output = (ms_filter_get_id(output) == MS_RTP_SEND_ID) ? TRUE : FALSE;
 	}
-	/* Define target upload bandwidth for RTCP packets sending. */
-	if (pt->normal_bitrate > 0) {
-		rtp_session_set_target_upload_bandwidth(stream->ms.sessions.rtp_session, pt->normal_bitrate);
-	}
+	
 	if (media_stream_get_direction(&stream->ms) == MediaStreamSendRecv || media_stream_get_direction(&stream->ms) == MediaStreamRecvOnly) {
 		MSConnectionHelper ch;
 
@@ -1407,9 +1441,12 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 	bool_t use_player        = (sink && !stream->player_active) || (!sink && stream->player_active);
 	bool_t change_source       = ( cam!=stream->cam || new_src_different || use_player);
 	bool_t encoder_has_builtin_converter = (!stream->pixconv && !stream->sizeconv);
+	/* We get the ticker from the source filter rather than stream->ms.sessions.ticker, for the case where the graph is actually running
+	 * in a MSVideoConference that has its own ticker. */
+	MSTicker *current_ticker = stream->source ? ms_filter_get_ticker(stream->source) : NULL;
 
-	if (stream->ms.sessions.ticker && stream->source){
-		ms_ticker_detach(stream->ms.sessions.ticker,stream->source);
+	if (current_ticker && stream->source){
+		ms_ticker_detach(current_ticker, stream->source);
 		/*unlink source filters and subsequent post processing filters */
 		if (encoder_has_builtin_converter || (stream->source_performs_encoding == TRUE)) {
 			ms_filter_unlink(stream->source, 0, stream->tee, 0);
@@ -1419,13 +1456,15 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 			if (stream->sizeconv) {
 				ms_filter_unlink (stream->tee, 0, stream->sizeconv, 0);
 				if (stream->source_performs_encoding == FALSE) {
-					ms_filter_unlink(stream->sizeconv, 0, stream->ms.encoder, 0);
+					if( stream->ms.encoder)
+						ms_filter_unlink(stream->sizeconv, 0, stream->ms.encoder, 0);
 				} else {
 					ms_filter_unlink(stream->sizeconv, 0, stream->ms.rtpsend, 0);
 				}
 			} else {
 				if (stream->source_performs_encoding == FALSE) {
-					ms_filter_unlink(stream->tee, 0, stream->ms.encoder, 0);
+					if(stream->ms.encoder)
+						ms_filter_unlink(stream->tee, 0, stream->ms.encoder, 0);
 				} else {
 					ms_filter_unlink(stream->tee, 0, stream->ms.rtpsend, 0);
 				}
@@ -1466,7 +1505,7 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 			ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_SET_DEVICE_ORIENTATION,&stream->device_orientation);
 		}
 
-		if (!skip_payload_config) {
+		if (!skip_payload_config && stream->ms.sessions.rtp_session) {
 			PayloadType *pt;
 			RtpProfile *profile;
 			int payload;
@@ -1483,7 +1522,7 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 			if (!skip_bitrate) apply_bitrate_limit(stream ,pt);
 		}
 
-		configure_video_source(stream, skip_bitrate, FALSE);
+		configure_video_source(stream, skip_bitrate, change_source);
 
 		if (encoder_has_builtin_converter || (stream->source_performs_encoding == TRUE)) {
 			ms_filter_link (stream->source, 0, stream->tee, 0);
@@ -1494,13 +1533,15 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 			if (stream->sizeconv) {
 				ms_filter_link (stream->tee, 0, stream->sizeconv, 0);
 				if (stream->source_performs_encoding == FALSE) {
-					ms_filter_link(stream->sizeconv, 0, stream->ms.encoder, 0);
+					if(stream->ms.encoder)
+						ms_filter_link(stream->sizeconv, 0, stream->ms.encoder, 0);
 				} else {
 					ms_filter_link(stream->sizeconv, 0, stream->ms.rtpsend, 0);
 				}
 			} else {
 				if (stream->source_performs_encoding == FALSE) {
-					ms_filter_link(stream->tee, 0, stream->ms.encoder, 0);
+					if(stream->ms.encoder)
+						ms_filter_link(stream->tee, 0, stream->ms.encoder, 0);
 				} else {
 					ms_filter_link(stream->tee, 0, stream->ms.rtpsend, 0);
 				}
@@ -1508,11 +1549,11 @@ static MSFilter* _video_stream_change_camera(VideoStream *stream, MSWebCam *cam,
 		}
 
 		// Change value of mirroring as potentially the webcam was changed
-		if (stream->output2 && ms_filter_get_id(stream->output2) == MS_OGL_ID) {
+		if (stream->output2 && ms_filter_implements_interface(stream->output2, MSFilterVideoDisplayInterface) ) {
 			assign_value_to_mirroring_flag_to_preview(stream);
 		}
 
-		ms_ticker_attach(stream->ms.sessions.ticker,stream->source);
+		ms_ticker_attach(current_ticker, stream->source);
 	}
 	return old_source;
 }
@@ -2045,6 +2086,9 @@ static MSFilter* _video_preview_change_camera(VideoPreview *stream, MSWebCam *ca
 		if (stream->tee) {
 			ms_connection_helper_link(&ch, stream->tee, 0, 0);
 			if (stream->output2) {
+				if( ms_filter_implements_interface(stream->output2, MSFilterVideoDisplayInterface) ) {
+				    assign_value_to_mirroring_flag_to_preview(stream);
+				}
 				ms_filter_link(stream->tee, 1, stream->output2, 0);
 			}
 			if (stream->local_jpegwriter) {
@@ -2095,8 +2139,9 @@ void video_stream_send_only_stop(VideoStream *vs){
 void video_stream_enable_zrtp(VideoStream *vstream, AudioStream *astream){
 	if (astream->ms.sessions.zrtp_context != NULL && vstream->ms.sessions.zrtp_context == NULL) {
 		vstream->ms.sessions.zrtp_context=ms_zrtp_multistream_new(&(vstream->ms.sessions), astream->ms.sessions.zrtp_context);
-	} else if (vstream->ms.sessions.zrtp_context && !media_stream_secured(&vstream->ms))
+	} else if (vstream->ms.sessions.zrtp_context && !media_stream_secured(&vstream->ms)) {
 		ms_zrtp_reset_transmition_timer(vstream->ms.sessions.zrtp_context);
+	}
 }
 
 void video_stream_start_zrtp(VideoStream *stream) {
