@@ -626,15 +626,17 @@ static void notify_frame_error_if_any(Vp8RtpFmtUnpackerCtx *ctx, Vp8RtpFmtFrame
 	}
 }
 
-static void add_frame(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t **packets_list) {
+static void add_frame(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t **packets_list, bool_t end_missing) {
 	Vp8RtpFmtFrame *frame;
 
 	if (*packets_list != NULL) {
 		frame = ms_new0(Vp8RtpFmtFrame, 1);
 		generate_frame_partitions_list(frame, *packets_list);
 		check_frame_partitions_list(ctx, frame);
+		if (end_missing) mark_frame_as_invalid(ctx, frame);
 		notify_frame_error_if_any(ctx, frame);
 		ctx->frames_list = bctbx_list_append(ctx->frames_list, (void *)frame);
+		frame->timestamp = ctx->last_ts;
 		bctbx_list_free(*packets_list);
 	}
 	*packets_list = NULL;
@@ -660,7 +662,7 @@ static void generate_frames_list(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t *packet
 		if ((ctx->initialized_last_ts == TRUE) && (ts != ctx->last_ts)) {
 			/* The current packet is from a frame different than the previous one
 			* (that apparently is not complete). */
-			add_frame(ctx, &frame_packets_list);
+			add_frame(ctx, &frame_packets_list, TRUE);
 		}
 		ctx->last_ts = ts;
 		ctx->initialized_last_ts = TRUE;
@@ -670,7 +672,7 @@ static void generate_frames_list(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t *packet
 
 		if (mblk_get_marker_info(packet->m)) {
 			/* The current packet is the last of the current frame. */
-			add_frame(ctx, &frame_packets_list);
+			add_frame(ctx, &frame_packets_list, FALSE);
 		}
 	}
 
@@ -698,7 +700,8 @@ static void output_frame(MSQueue *out, Vp8RtpFmtFrame *frame) {
 	if (om != NULL) {
 		if (om->b_cont) msgpullup(om, -1);
 		mblk_set_marker_info(om, 1);
-		ms_queue_put(out, (void *)om);
+		mblk_set_timestamp_info(om, frame->timestamp);
+		ms_queue_put(out, om);
 	}
 }
 
@@ -710,7 +713,7 @@ static void output_partition(MSQueue *out, Vp8RtpFmtPartition **partition, bool_
 	if ((*partition)->has_marker || (last == TRUE)) {
 		mblk_set_marker_info((*partition)->m, 1);
 	}
-	ms_queue_put(out, (void *)(*partition)->m);
+	ms_queue_put(out, (*partition)->m);
 	(*partition)->outputted = TRUE;
 }
 
@@ -783,7 +786,7 @@ static int output_valid_partitions(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out) {
 						if (ctx->waiting_for_reference_frame_count > max_non_reference_frame_after_sli_or_pli){
 							/* Workaround/security: if SLI has no effect after a certain number of received frames, use PLI. */
 							ms_warning("VP8 decoder: requesting PLI, reference frame still not received.");
-							if (ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_SEND_PLI);
+							send_pli(ctx);
 						}else if (ctx->sli_pending){
 							/* retransmit last SLI */
 							if (ctx->filter) ms_filter_notify(ctx->filter, MS_VIDEO_DECODER_SEND_SLI, &ctx->current_sli);
@@ -840,8 +843,8 @@ static void free_partitions_of_frame(void *data) {
 }
 
 static void clean_frame(Vp8RtpFmtUnpackerCtx *ctx) {
-	if (bctbx_list_size(ctx->frames_list) > 0) {
-		Vp8RtpFmtFrame *frame = bctbx_list_nth_data(ctx->frames_list, 0);
+	if (ctx->frames_list) {
+		Vp8RtpFmtFrame *frame = (Vp8RtpFmtFrame*)bctbx_list_nth_data(ctx->frames_list, 0);
 		free_partitions_of_frame(frame);
 		ctx->frames_list = bctbx_list_remove(ctx->frames_list, frame);
 		ms_free(frame);
@@ -921,6 +924,60 @@ static Vp8RtpFmtErrorCode parse_payload_descriptor(Vp8RtpFmtPacket *packet) {
 	return Vp8RtpFmtOk;
 }
 
+uint8_t * vp8rtpfmt_skip_payload_descriptor(mblk_t *m){
+	uint8_t *h = m->b_rptr;
+	unsigned int packet_size = (unsigned int)(m->b_wptr - m->b_rptr);
+	uint8_t offset = 0;
+	bool_t extended_control_bits_present = FALSE;
+	bool_t pictureid_present = FALSE;
+	bool_t tid_present = FALSE;
+	bool_t keyidx_present = FALSE;
+	bool_t tl0picidx_present = FALSE;
+
+	if (packet_size == 0) return NULL;
+
+	/* Parse mandatory first octet of payload descriptor. */
+	if (h[offset] & (1 << 7)) extended_control_bits_present = TRUE;
+	
+	offset++;
+	if (offset >= packet_size) return NULL;
+	/* Parse the first extension octet if needed. */
+	if (extended_control_bits_present == TRUE) {
+		if (h[offset] & (1 << 7)) pictureid_present = TRUE;
+		if (h[offset] & (1 << 6)) tl0picidx_present = TRUE;
+		if (h[offset] & (1 << 5)) tid_present = TRUE;
+		if (h[offset] & (1 << 4)) keyidx_present = TRUE;
+		if ((tl0picidx_present == TRUE) && (tid_present != TRUE)) {
+			/* Invalid payload descriptor. */
+			return NULL;
+		}
+		offset++;
+		if (offset >= packet_size) return NULL;
+	}
+	/* Parse the pictureID if needed. */
+	if (pictureid_present == TRUE) {
+		if (h[offset] & (1 << 7)) {
+			/* The pictureID is 16 bits long. */
+			if ((unsigned int)(offset + 1) >= packet_size) return NULL;
+			offset += 2;
+		} else {
+			/* The pictureId is 8 bits long. */
+			offset++;
+		}
+		if (offset >= packet_size) return NULL;
+	}
+	/* Parse the tl0picidx if needed. */
+	if (tl0picidx_present == TRUE) {
+		offset++;
+		if (offset >= packet_size) return NULL;
+	}
+	if ((tid_present == TRUE) || (keyidx_present == TRUE)) {
+		offset++;
+		if (offset >= packet_size) return NULL;
+	}
+	return &h[offset];
+}
+
 
 void vp8rtpfmt_unpacker_init(Vp8RtpFmtUnpackerCtx *ctx, MSFilter *f, bool_t avpf_enabled, bool_t freeze_on_error, bool_t output_partitions) {
 	ctx->filter = f;
@@ -968,7 +1025,7 @@ void vp8rtpfmt_unpacker_feed(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *in) {
 			ctx->ref_cseq++;
 			if (ctx->ref_cseq!=cseq){
 				packet->cseq_inconsistency=TRUE;
-				ms_message("sequence inconsistency detected (diff=%i)",(int)(cseq-ctx->ref_cseq));
+				ms_message("Vp8RtpFmtUnpackerCtx filter=%p: sequence inconsistency detected (cseq=%u, diff=%i) m=%p", ctx->filter, (unsigned int) cseq, (int)(cseq-ctx->ref_cseq), m);
 				ctx->ref_cseq=cseq;
 			}
 		}
