@@ -84,6 +84,7 @@ extern "C"{
 		Vp8RtpFmtFramePartitionsInfo partitions_info;
 		Vp8RtpFmtPartition *partitions[9];
 		Vp8RtpFmtErrorCode error;
+		uint32_t timestamp;
 		uint16_t pictureid;
 		bool_t pictureid_present;
 		bool_t unnumbered_partitions;
@@ -136,6 +137,9 @@ extern "C"{
 	int vp8rtpfmt_unpacker_get_frame(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out, Vp8RtpFmtFrameInfo *frame_info);
 	uint32_t vp8rtpfmt_unpacker_calc_extended_cseq(Vp8RtpFmtUnpackerCtx *ctx, uint16_t cseq);
 	void vp8rtpfmt_send_rpsi(Vp8RtpFmtUnpackerCtx *ctx, uint16_t pictureid);
+	
+	/* Fast version that just skips the vp8 rtp payload header, and returns the start of the VP8 payload. */
+	uint8_t * vp8rtpfmt_skip_payload_descriptor(mblk_t *m);
 
 #ifdef __cplusplus
 }
